@@ -1,9 +1,9 @@
 /*****************************************************************************
  * dvb.c: linux-dvb input for DVBlast
  *****************************************************************************
- * Copyright (C) 2008-2010, 2015 VideoLAN
+ * Copyright (C) 2008-2010, 2015, 2025 VideoLAN
  *
- * Authors: Christophe Massiot <massiot@via.ecp.fr>
+ * Authors: Christophe Massiot <cmassiot@upipe.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -72,7 +72,7 @@
 
 int i_dvr_buffer_size = DVR_BUFFER_SIZE;
 
-static int i_frontend, i_dvr;
+static int i_frontend, i_dvr, i_sec;
 static struct ev_io frontend_watcher, dvr_watcher;
 static struct ev_timer lock_watcher, mute_watcher, print_watcher;
 static fe_status_t i_last_status;
@@ -87,6 +87,59 @@ static void FrontendRead(struct ev_loop *loop, struct ev_io *w, int revents);
 static void FrontendLockCb(struct ev_loop *loop, struct ev_timer *w, int revents);
 static void FrontendSet( bool b_reset );
 
+/*****************************************************************************
+ * dvb_OpenDvr
+ *****************************************************************************/
+void dvb_OpenDvr( void )
+{
+    char psz_tmp[128];
+
+    sprintf( psz_tmp, "/dev/dvb/adapter%d/dvr%d", i_adapter, i_fenum );
+    if( (i_dvr = open(psz_tmp, O_RDONLY | O_NONBLOCK)) < 0 )
+    {
+        msg_Info(NULL, "opening device %s failed (%s), fallback on sec device",
+                 psz_tmp, strerror(errno));
+
+        /* try to read from sec device */
+        sprintf(psz_tmp, "/dev/dvb/adapter%d/sec%d", i_adapter, i_fenum);
+        if ( (i_dvr = open(psz_tmp, O_RDONLY | O_NONBLOCK)) < 0 )
+        {
+            msg_Err(NULL, "opening device %s failed (%s)", psz_tmp,
+                    strerror(errno));
+            exit(1);
+        }
+    }
+
+    if ( ioctl( i_dvr, DMX_SET_BUFFER_SIZE, i_dvr_buffer_size ) < 0 )
+    {
+        msg_Warn( NULL, "couldn't set %s buffer size (%s)", psz_tmp,
+                 strerror(errno) );
+    }
+
+    ev_io_init(&dvr_watcher, DVRRead, i_dvr, EV_READ);
+    ev_io_start(event_loop, &dvr_watcher);
+}
+
+/*****************************************************************************
+ * dvb_OpenSec
+ *****************************************************************************/
+void dvb_OpenSec( void )
+{
+    char psz_tmp[128];
+
+    msg_Dbg( NULL, "compiled with DVB API version %d.%d", DVB_API_VERSION, DVB_API_VERSION_MINOR );
+
+    i_frontend = -1;
+
+    sprintf( psz_tmp, "/dev/dvb/adapter%d/sec%d", i_adapter, i_secnum );
+    if( (i_sec = open(psz_tmp, O_WRONLY)) < 0 )
+    {
+        msg_Err(NULL, "opening device %s failed (%s)", psz_tmp,
+                strerror(errno));
+        exit(1);
+    }
+}
+
 /*****************************************************************************
  * dvb_Open
  *****************************************************************************/
@@ -113,23 +166,7 @@ void dvb_Open( void )
         i_frontend = -1;
     }
 
-    sprintf( psz_tmp, "/dev/dvb/adapter%d/dvr%d", i_adapter, i_fenum );
-
-    if( (i_dvr = open(psz_tmp, O_RDONLY | O_NONBLOCK)) < 0 )
-    {
-        msg_Err( NULL, "opening device %s failed (%s)", psz_tmp,
-                 strerror(errno) );
-        exit(1);
-    }
-
-    if ( ioctl( i_dvr, DMX_SET_BUFFER_SIZE, i_dvr_buffer_size ) < 0 )
-    {
-        msg_Warn( NULL, "couldn't set %s buffer size (%s)", psz_tmp,
-                 strerror(errno) );
-    }
-
-    ev_io_init(&dvr_watcher, DVRRead, i_dvr, EV_READ);
-    ev_io_start(event_loop, &dvr_watcher);
+    dvb_OpenDvr();
 
     if ( i_frontend != -1 )
     {
@@ -271,6 +308,33 @@ void dvb_UnsetFilter( int i_fd, uint16_t i_pid )
     close( i_fd );
 }
 
+/*****************************************************************************
+ * dvb_WriteSec: write data to sec device
+ *****************************************************************************/
+void dvb_WriteSec( block_t *p_ts )
+{
+    int i_iov = 0;
+
+    for (block_t *p_b = p_ts; p_b; p_b = p_b->p_next)
+        i_iov++;
+
+    struct iovec p_iov[i_iov];
+    struct iovec *p_current = &p_iov[0];
+    for (block_t *p_b = p_ts; p_b; p_b = p_b->p_next)
+    {
+        p_current->iov_base = p_b->p_ts;
+        p_current->iov_len = TS_SIZE;
+        p_current++;
+    }
+
+    int i_len;
+    if ( (i_len = writev( i_sec, p_iov, i_iov )) < 0 )
+    {
+        msg_Err( NULL, "couldn't write to DVB sec (%s)", strerror(errno) );
+    }
+
+    block_DeleteChain( p_ts );
+}
 
 /*
  * Frontend
@@ -290,16 +354,37 @@ static void PrintCb( struct ev_loop *loop, struct ev_timer *w, int revents )
     ioctl(i_frontend, FE_READ_SNR, &i_snr);
     ioctl(i_frontend, FE_READ_UNCORRECTED_BLOCKS, &i_uncorrected);
 
+    int64_t i_strength_dbm = i_strength;
+    int64_t i_snr_db = i_snr;
+
+#ifdef DTV_STAT_SIGNAL_STRENGTH
+    struct dtv_property prop[] = {
+        { .cmd = DTV_STAT_SIGNAL_STRENGTH },
+        { .cmd = DTV_STAT_CNR },
+    };
+
+    struct dtv_properties props = { .num = 2, .props = prop };
+    if (ioctl(i_frontend, FE_GET_PROPERTY, &props) != -1)
+    {
+        if (prop[0].u.st.len > 0 &&
+            prop[0].u.st.stat[0].scale == FE_SCALE_DECIBEL)
+            i_strength_dbm = prop[0].u.st.stat[0].svalue;
+        if (prop[1].u.st.len > 0 &&
+            prop[1].u.st.stat[0].scale == FE_SCALE_DECIBEL)
+            i_snr_db = prop[1].u.st.stat[0].svalue;
+    }
+#endif
+
     switch (i_print_type)
     {
         case PRINT_XML:
             fprintf(print_fh,
-                    "<STATUS type=\"frontend\" ber=\"%"PRIu32"\" strength=\"%"PRIu16"\" snr=\"%"PRIu16"\" uncorrected=\"%"PRIu32"\" />\n",
-                    i_ber, i_strength, i_snr, i_uncorrected);
+                    "<STATUS type=\"frontend\" ber=\"%"PRIu32"\" strength=\"%"PRId64"\" snr=\"%"PRId64"\" uncorrected=\"%"PRIu32"\" />\n",
+                    i_ber, i_strength_dbm, i_snr_db, i_uncorrected);
             break;
         case PRINT_TEXT:
-            fprintf(print_fh, "frontend ber: %"PRIu32" strength: %"PRIu16" snr: %"PRIu16" uncorrected: %"PRIu32"\n",
-                    i_ber, i_strength, i_snr, i_uncorrected);
+            fprintf(print_fh, "frontend ber: %"PRIu32" strength: %"PRId64" snr: %"PRId64" uncorrected: %"PRIu32"\n",
+                    i_ber, i_strength_dbm, i_snr_db, i_uncorrected);
             break;
         default:
             break;
@@ -742,6 +827,23 @@ static fe_rolloff_t GetRollOff(void)
     }
 }
 
+static int GetSSI(void)
+{
+    if ( streq(psz_mis_pls_mode, "GOLD") )
+        return i_mis_pls_code;
+
+    /* ROOT mode */
+    uint32_t x, g;
+
+    for ( g = 0, x = 1; g < 0x3ffff; g++ )
+    {
+        if ( i_mis_pls_code == x )
+            return g;
+        x = (((x ^ (x >> 7)) & 1) << 17) | (x >> 1);
+    }
+    return 0xffffffff;
+}
+
 static fe_guard_interval_t GetGuard(void)
 {
     switch ( i_guard )
@@ -921,19 +1023,21 @@ static struct dtv_properties dvbs_cmdseq = {
 #define IDX_DVBS2_PILOT     6
 #define IDX_DVBS2_ROLLOFF   7
 #define IDX_DVBS2_STREAM_ID 8
+#define IDX_DVBS2_SSI       9
 
 /* Commands 0..5 are the same as dvbs_cmdargs */
 /* Commands 6..8 are special for DVB-S2 */
 static struct dtv_property dvbs2_cmdargs[] = {
-    { .cmd = DTV_DELIVERY_SYSTEM, .u.data = SYS_DVBS2 },
-    { .cmd = DTV_FREQUENCY,       .u.data = 0 },
-    { .cmd = DTV_MODULATION,      .u.data = PSK_8 },
-    { .cmd = DTV_INVERSION,       .u.data = INVERSION_AUTO },
-    { .cmd = DTV_SYMBOL_RATE,     .u.data = 27500000 },
-    { .cmd = DTV_INNER_FEC,       .u.data = FEC_AUTO },
-    { .cmd = DTV_PILOT,           .u.data = PILOT_AUTO },   /* idx: 6 */
-    { .cmd = DTV_ROLLOFF,         .u.data = ROLLOFF_AUTO }, /* idx: 7 */
-    { .cmd = DTV_STREAM_ID,       .u.data = 0 },            /* idx: 8 */
+    { .cmd = DTV_DELIVERY_SYSTEM,           .u.data = SYS_DVBS2 },
+    { .cmd = DTV_FREQUENCY,                 .u.data = 0 },
+    { .cmd = DTV_MODULATION,                .u.data = PSK_8 },
+    { .cmd = DTV_INVERSION,                 .u.data = INVERSION_AUTO },
+    { .cmd = DTV_SYMBOL_RATE,               .u.data = 27500000 },
+    { .cmd = DTV_INNER_FEC,                 .u.data = FEC_AUTO },
+    { .cmd = DTV_PILOT,                     .u.data = PILOT_AUTO },   /* idx: 6 */
+    { .cmd = DTV_ROLLOFF,                   .u.data = ROLLOFF_AUTO }, /* idx: 7 */
+    { .cmd = DTV_STREAM_ID,                 .u.data = 0 },            /* idx: 8 */
+    { .cmd = DTV_SCRAMBLING_SEQUENCE_INDEX, .u.data = 0 },            /* idx: 9 */
     { .cmd = DTV_TUNE },
 };
 static struct dtv_properties dvbs2_cmdseq = {
@@ -1309,7 +1413,12 @@ static void FrontendSet( bool b_init )
             p->props[MODULATION].u.data = GetModulation();
             p->props[IDX_DVBS2_PILOT].u.data = GetPilot();
             p->props[IDX_DVBS2_ROLLOFF].u.data = GetRollOff();
-            p->props[IDX_DVBS2_STREAM_ID].u.data = i_mis;
+            if ( i_mis_pls_code )
+            {
+                p->props[IDX_DVBS2_STREAM_ID].u.data =
+                    i_mis_is_id != -1 ? i_mis_is_id : 0x80000000;
+                p->props[IDX_DVBS2_SSI].u.data = GetSSI();
+            }
         }
         else
             p = &dvbs_cmdseq;
@@ -1319,10 +1428,16 @@ static void FrontendSet( bool b_init )
         p->props[FEC_INNER].u.data = GetFECInner(info.caps);
         p->props[FREQUENCY].u.data = FrontendDoDiseqc();
 
-        msg_Dbg( NULL, "tuning DVB-S frontend to f=%d srate=%d inversion=%d fec=%d rolloff=%d modulation=%s pilot=%d mis=%d /pls-mode: %s (%d) pls-code: %d is-id: %d /",
-                 i_frequency, i_srate, i_inversion, i_fec, i_rolloff,
-                 psz_modulation == NULL ? "legacy" : psz_modulation, i_pilot,
-                 i_mis, psz_mis_pls_mode, i_mis_pls_mode, i_mis_pls_code, i_mis_is_id );
+        if ( i_mis_is_id == -1 )
+            msg_Dbg( NULL, "tuning DVB-S frontend in BBFrame mode to f=%d srate=%d inversion=%d fec=%d rolloff=%d modulation=%s pilot=%d pls-mode: %s pls-code: %d",
+                    i_frequency, i_srate, i_inversion, i_fec, i_rolloff,
+                    psz_modulation == NULL ? "legacy" : psz_modulation, i_pilot,
+                    psz_mis_pls_mode, i_mis_pls_code );
+        else
+            msg_Dbg( NULL, "tuning DVB-S frontend to f=%d srate=%d inversion=%d fec=%d rolloff=%d modulation=%s pilot=%d pls-mode: %s pls-code: %d is-id: %u",
+                    i_frequency, i_srate, i_inversion, i_fec, i_rolloff,
+                    psz_modulation == NULL ? "legacy" : psz_modulation, i_pilot,
+                    psz_mis_pls_mode, i_mis_pls_code, i_mis_is_id );
         break;
 
     case SYS_ATSC:
