@@ -1,9 +1,9 @@
 /*****************************************************************************
  * dvblast.c
  *****************************************************************************
- * Copyright (C) 2004, 2008-2011, 2015 VideoLAN
+ * Copyright (C) 2004, 2008-2011, 2015, 2020, 2025 VideoLAN
  *
- * Authors: Christophe Massiot <massiot@via.ecp.fr>
+ * Authors: Christophe Massiot <cmassiot@upipe.org>
  *          Andy Gatward <a.j.gatward@reading.ac.uk>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -65,6 +65,7 @@ static int i_priority = -1;
 int i_adapter = 0;
 int i_fenum = 0;
 int i_canum = 0;
+int i_secnum = -1;
 char *psz_delsys = NULL;
 int i_frequency = 0;
 char *psz_lnb_type = "universal";
@@ -80,9 +81,7 @@ int b_tone = 0;
 int i_bandwidth = 8;
 char *psz_modulation = NULL;
 int i_pilot = -1;
-int i_mis = 0;
 char *psz_mis_pls_mode = "ROOT";
-int i_mis_pls_mode = 0;
 int i_mis_pls_code = 0;
 int i_mis_is_id = 0;
 int i_fec_lp = 999;
@@ -103,6 +102,7 @@ bool b_print_enabled = false;
 FILE *print_fh;
 mtime_t i_print_period = 0;
 mtime_t i_es_timeout = 0;
+mtime_t i_udp_lock_timeout = DEFAULT_UDP_LOCK_TIMEOUT;
 
 int i_verbose = DEFAULT_VERBOSITY;
 int i_syslog = 0;
@@ -157,6 +157,8 @@ void config_Init( output_config_t *p_config )
     p_config->i_if_index_v6 = -1;
     p_config->i_srcport = 0;
 
+    if ( i_mis_is_id == -1 )
+        p_config->i_sid = UINT16_MAX; /* sid is mis */
     p_config->pi_pids = NULL;
     p_config->b_passthrough = false;
     p_config->b_do_remap = false;
@@ -241,7 +243,7 @@ static uint8_t *config_striconv( const char *psz_string,
     size_t i_available = i_output;
     char *p_output = malloc( i_output );
     char *p = p_output;
-    if ( iconv( conf_iconv, &psz_tmp, &i_input, &p, &i_available ) == -1 )
+    if ( iconv( conf_iconv, &psz_tmp, &i_input, &p, &i_available ) == (size_t)-1 )
     {
         free( p_output );
 
@@ -632,9 +634,9 @@ void usage()
         "[-u] [-w] [-U] [-L <latency>] [-E <retention>] [-d <dest IP>[<:port>][/<opts>]*] [-3] "
         "[-z] [-C [-e] [-M <network name>] [-N <network ID>]] [-T] [-j <system charset>] "
         "[-W] [-Y] [-l] [-g <logger ident>] [-Z <mrtg file>] [-V] [-h] [-B <provider_name>] "
-        "[-1 <mis_id>] [-2 <size>] [-5 <DVBS|DVBS2|DVBC_ANNEX_A|DVBC_ANNEX_B|DVBT|DVBT2|ATSC|ISDBT>] -y <ca_dev_number> "
+        "[-1] [-2 <size>] [-5 <DVBS|DVBS2|DVBC_ANNEX_A|DVBC_ANNEX_B|DVBT|DVBT2|ATSC|ISDBT>] -y <ca_dev_number> "
         "[-J <DVB charset>] [-Q <quit timeout>] [-0 pid_mapping] [-x <text|xml>]"
-        "[-6 <print period>] [-7 <ES timeout>]" );
+        "[-6 <print period>] [-7 <ES timeout>] [-4 <UDP lock timeout>]" );
 
     msg_Raw( NULL, "Input:" );
 #ifdef HAVE_ASI_SUPPORT
@@ -660,11 +662,12 @@ void usage()
     msg_Raw( NULL, "    DVB-T  qam_16|qam_32|qam_64|qam_128|qam_256 (default qam_auto)" );
     msg_Raw( NULL, "    DVB-S2 qpsk|psk_8|apsk_16|apsk_32 (default legacy DVB-S)" );
     msg_Raw( NULL, "  -n --frontend-number <frontend number>" );
+    msg_Raw( NULL, "     --sec-number <sec number>" );
     msg_Raw( NULL, "  -p --force-pulse      force 22kHz pulses for high-band selection (DVB-S)" );
     msg_Raw( NULL, "  -P --pilot            DVB-S2 Pilot (-1 auto, 0 off, 1 on)" );
     msg_Raw( NULL, "  -R --rolloff          DVB-S2 Rolloff value" );
     msg_Raw( NULL, "    DVB-S2 35=0.35|25=0.25|20=0.20|0=AUTO (default: 35)" );
-    msg_Raw( NULL, "  -1 --multistream-id   Set stream ID (0-2147483648, default: 0)." );
+    msg_Raw( NULL, "  -1 --bbframe-demux    use raw BBframes to demultiplex MIS" );
     msg_Raw( NULL, "     --multistream-id-pls-mode   Set multistream PLS mode (ROOT, GOLD, COMBO, default: ROOT)" );
     msg_Raw( NULL, "     --multistream-id-pls-code   Set multistream PLS code (0-262143, default: 0)" );
     msg_Raw( NULL, "     --multistream-id-is-id      Set multistream IS id (0-255, default: 0)" );
@@ -716,6 +719,7 @@ void usage()
     msg_Raw( NULL, "  -Q --quit-timeout     when locked, quit after this delay (in ms), or after the first lock timeout" );
     msg_Raw( NULL, "  -6 --print-period     periodicity at which we print bitrate and errors (in ms)" );
     msg_Raw( NULL, "  -7 --es-timeout       time of inactivy before which a PID is reported down (in ms)" );
+    msg_Raw( NULL, "  -4 --udp lock-timeout time of inactivy before which a UDP stream is reported down (in ms)" );
     msg_Raw( NULL, "  -r --remote-socket <remote socket>" );
     msg_Raw( NULL, "  -Z --mrtg-file <file> Log input packets and errors into mrtg-file" );
     msg_Raw( NULL, "  -V --version          only display the version" );
@@ -740,8 +744,7 @@ int main( int i_argc, char **pp_argv )
         usage();
 
     /*
-     * The only short options left are: 4
-     * Use them wisely.
+     * No short options are left.
      */
     static const struct option long_options[] =
     {
@@ -752,6 +755,7 @@ int main( int i_argc, char **pp_argv )
         { "priority",        required_argument, NULL, 'i' },
         { "adapter",         required_argument, NULL, 'a' },
         { "frontend-number", required_argument, NULL, 'n' },
+        { "sec-number",      required_argument, NULL, 0x100004 },
         { "delsys",          required_argument, NULL, '5' },
         { "dvb-plp-id",      required_argument, NULL, '9' },
         { "frequency",       required_argument, NULL, 'f' },
@@ -767,7 +771,7 @@ int main( int i_argc, char **pp_argv )
         { "inversion",       required_argument, NULL, 'I' },
         { "modulation",      required_argument, NULL, 'm' },
         { "pilot",           required_argument, NULL, 'P' },
-        { "multistream-id",  required_argument, NULL, '1' },
+        { "bbframe-demux",   no_argument,       NULL, '1' },
         { "multistream-id-pls-mode",  required_argument, NULL, 0x100001 },
         { "multistream-id-pls-code",  required_argument, NULL, 0x100002 },
         { "multistream-id-is-id"   ,  required_argument, NULL, 0x100003 },
@@ -802,6 +806,7 @@ int main( int i_argc, char **pp_argv )
         { "quit-timeout",    required_argument, NULL, 'Q' },
         { "print-period",    required_argument, NULL, '6' },
         { "es-timeout",      required_argument, NULL, '7' },
+        { "udp-lock-timeout", required_argument, NULL, '4' },
         { "quiet",           no_argument,       NULL, 'q' },
         { "help",            no_argument,       NULL, 'h' },
         { "version",         no_argument,       NULL, 'V' },
@@ -812,7 +817,7 @@ int main( int i_argc, char **pp_argv )
         { 0, 0, 0, 0 }
     };
 
-    while ( (c = getopt_long(i_argc, pp_argv, "q::c:r:t:o:i:a:n:5:f:F:R:s:S:k:v:pb:I:m:P:K:G:H:X:O:uwUTL:E:d:3D:A:lg:zCWYeM:N:j:J:B:x:Q:6:7:hVZ:y:0:1:2:9:", long_options, NULL)) != -1 )
+    while ( (c = getopt_long(i_argc, pp_argv, "q::c:r:t:o:i:a:n:5:f:F:R:s:S:k:v:pb:I:m:P:K:G:H:X:O:uwUTL:E:d:3D:A:lg:zCWYeM:N:j:J:B:x:Q:6:7:4:hVZ:y:0:12:9:", long_options, NULL)) != -1 )
     {
         switch ( c )
         {
@@ -878,6 +883,10 @@ int main( int i_argc, char **pp_argv )
             i_fenum = strtol( optarg, NULL, 0 );
             break;
 
+        case 0x100004:
+            i_secnum = strtol( optarg, NULL, 0 );
+            break;
+
         case 'y':
             i_canum = strtol( optarg, NULL, 0 );
             break;
@@ -885,9 +894,11 @@ int main( int i_argc, char **pp_argv )
         case '5':
             psz_delsys = optarg;
             break;
+
         case '9':
             dvb_plp_id = strtol( optarg, NULL, 0 );
             break;
+
         case 'f':
             if (optarg && optarg[0] != '-')
                 i_frequency = strtol( optarg, NULL, 0 );
@@ -953,34 +964,41 @@ int main( int i_argc, char **pp_argv )
             break;
 
         case '1':
-            i_mis = strtol( optarg, NULL, 0 );
+            i_mis_is_id = -1;
+            /* intended pass-through */
+
+        case 'u':
+            b_budget_mode = 1;
             break;
 
         case 0x100001: // --multistream-id-pls-mode
             psz_mis_pls_mode = optarg;
-            if        ( streq( psz_mis_pls_mode, "ROOT" ) ) {
-                i_mis_pls_mode = 0;
-            } else if ( streq( psz_mis_pls_mode, "GOLD" ) ) {
-                i_mis_pls_mode = 1;
-            } else if ( streq( psz_mis_pls_mode, "COMBO" ) ) {
-                i_mis_pls_mode = 2;
-            } else {
-                msg_Err(NULL, "Invalid --multistream-id-pls-mode '%s', valid options are: ROOT GOLD COMBO", optarg);
+            if ( !streq(psz_mis_pls_mode, "ROOT") &&
+                 !streq(psz_mis_pls_mode, "GOLD") )
+            {
+                msg_Err(NULL, "Invalid --multistream-id-pls-mode '%s', valid options are: ROOT GOLD", optarg);
                 exit(1);
             }
             break;
 
         case 0x100002: // --multistream-id-pls-code
             i_mis_pls_code = strtol( optarg, NULL, 0 );
-            if ( i_mis_pls_code < 0 || i_mis_pls_code > 262143 ) {
+            if ( i_mis_pls_code < 0 || i_mis_pls_code > 262143 )
+            {
                 msg_Err(NULL, "ERROR: Invalid --multistream-id-pls-code '%s', valid options are: 0-262143", optarg);
                 exit(1);
             }
             break;
 
         case 0x100003: // --multistream-id-is-id
+            if ( i_mis_is_id == -1 )
+            {
+                msg_Err(NULL, "ERROR: --multistream-id-is-id is incompatible with --bbframe-demux");
+                exit(1);
+            }
             i_mis_is_id = strtol( optarg, NULL, 0 );
-            if ( i_mis_is_id < 0 || i_mis_is_id > 255 ) {
+            if ( i_mis_is_id < 0 || i_mis_is_id > 255 )
+            {
                 msg_Err(NULL, "ERROR: Invalid --multistream-id-is-id '%s', valid options are: 0-255", optarg);
                 exit(1);
             }
@@ -1006,10 +1024,6 @@ int main( int i_argc, char **pp_argv )
             i_hierarchy = strtol( optarg, NULL, 0 );
             break;
 
-        case 'u':
-            b_budget_mode = 1;
-            break;
-
         case 'w':
             b_select_pmts = !b_select_pmts;
             break;
@@ -1153,6 +1167,10 @@ int main( int i_argc, char **pp_argv )
             i_es_timeout = strtoll( optarg, NULL, 0 ) * 1000;
             break;
 
+        case '4':
+            i_udp_lock_timeout = strtoll( optarg, NULL, 0 ) * 1000;
+            break;
+
         case 'V':
             DisplayVersion();
             exit(0);
@@ -1221,7 +1239,10 @@ int main( int i_argc, char **pp_argv )
     {
         case PRINT_XML:
             fprintf(print_fh, "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n");
-            fprintf(print_fh, "<TS>\n");
+            if ( i_mis_is_id != -1 )
+                fprintf(print_fh, "<TS>\n");
+            else
+                fprintf(print_fh, "<MIS>\n");
             break;
         default:
             break;
@@ -1272,26 +1293,6 @@ int main( int i_argc, char **pp_argv )
 
     srand( time(NULL) * getpid() );
 
-    if ( i_mis_pls_mode || i_mis_pls_code || i_mis_is_id )
-    {
-        i_mis = calc_multistream_id( i_mis_pls_mode, i_mis_pls_code, i_mis_is_id );
-        msg_Info( NULL, "Calculating multistream-id using pls-mode: %s (%d) pls-code: %d is-id: %d. Resulting multistream-id: %d (0x%x)",
-            psz_mis_pls_mode, i_mis_pls_mode, i_mis_pls_code, i_mis_is_id, i_mis, i_mis );
-    }
-    else if ( i_mis )
-    {
-        i_mis_pls_mode = (i_mis >> 26) & 0x03;
-        i_mis_pls_code = (i_mis >> 8) & 0x3ffff;
-        i_mis_is_id    = i_mis & 0xff;
-        psz_mis_pls_mode =
-            i_mis_pls_mode == 0 ? "ROOT" :
-            i_mis_pls_mode == 1 ? "GOLD" :
-            i_mis_pls_mode == 2 ? "COMBO" : "UNKNOWN";
-
-        msg_Info( NULL, "Calculated multistream pls-mode: %s (%d) pls-code: %d is-id: %d from multistream-id: %d (0x%x)",
-            psz_mis_pls_mode, i_mis_pls_mode, i_mis_pls_code, i_mis_is_id, i_mis, i_mis );
-    }
-
     demux_Open();
 
     // init the mrtg logfile
@@ -1336,7 +1337,10 @@ int main( int i_argc, char **pp_argv )
     switch (i_print_type)
     {
     case PRINT_XML:
-        fprintf(print_fh, "</TS>\n");
+        if ( i_mis_is_id != -1 )
+            fprintf(print_fh, "</TS>\n");
+        else
+            fprintf(print_fh, "</MIS>\n");
         break;
     default:
         break;
